from django.contrib import admin

from wallet.models.wallet import Wallet


@admin.register(Wallet)
class WalletAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "name",
        "balance",
        "group",
        "created_at",
        "updated_at",
    )
    list_filter = (
        "created_at",
        "updated_at",
    )
    search_fields = ("name",)
    date_hierarchy = "created_at"



from rest_framework.serializers import ModelSerializer
from rest_framework import serializers
from rest_framework.validators import UniqueTogetherValidator, UniqueValidator

from wallet.models.currency import Currency, CurrencyRate


class CurrencySerializer(ModelSerializer):
    code = serializers.CharField(
        validators=(UniqueValidator(queryset=Currency.objects.all()),),
        required=True,
    )
    name = serializers.CharField(
        validators=(UniqueValidator(queryset=Currency.objects.all()),),
        required=True,
    )

    class Meta:
        model = Currency
        fields = ("id", "code", "name", "created_at", "updated_at")
        validators = (
            UniqueTogetherValidator(
                queryset=Currency.objects.all(),
                fields=(
                    "code",
                    "name",
                ),
            ),
        )


class CurrencyRateSerializer(ModelSerializer):
    class Meta:
        model = CurrencyRate
        fields = ("id", "currency", "rate")



# coding: utf-8

"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from openapi_client.models.patched_income import PatchedIncome  # noqa: E501

class TestPatchedIncome(unittest.TestCase):
    """PatchedIncome unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PatchedIncome:
        """Test PatchedIncome
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PatchedIncome`
        """
        model = PatchedIncome()  # noqa: E501
        if include_optional:
            return PatchedIncome(
                id = 56,
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                is_deleted = True,
                amount = '-80728880',
                comment = '',
                category = 56,
                created_by = 56,
                wallet = 56,
                group = 56
            )
        else:
            return PatchedIncome(
        )
        """

    def testPatchedIncome(self):
        """Test PatchedIncome"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



from django.db import models

from wallet.models.mixins import TimestampMixin, SafeDeletionMixin, SafeDeletionManager
from users.models import APIUser


class Group(TimestampMixin, SafeDeletionMixin):
    members = models.ManyToManyField(
        APIUser,
        related_name="groups",
        verbose_name="Group members",
        help_text="Members of the group",
        db_index=True,
    )
    objects = SafeDeletionManager()

    class Meta:
        verbose_name = "Group"
        verbose_name_plural = "Groups"

    def __str__(self):
        return str(self.pk)



from rest_framework.viewsets import ModelViewSet

from api.serializers import GroupSerializer
from wallet.models.group import Group


class GroupViewSet(ModelViewSet):
    queryset = Group.objects.order_by("pk")
    serializer_class = GroupSerializer



from django.urls import path
from django.views.decorators.csrf import csrf_exempt
from rest_framework.routers import SimpleRouter

from drf_spectacular.views import (
    SpectacularAPIView,
    SpectacularRedocView,
    SpectacularSwaggerView,
)

from api.views import (
    CurrencyViewSet,
    CurrencyRateViewSet,
    APIUserViewSet,
    ExpenseCategoryViewSet,
    ExpenseViewSet,
    get_token,
    GroupViewSet,
    IncomeCategoryViewSet,
    IncomeViewSet,
    signin,
    signup,
    TransferViewSet,
    WalletViewSet,
)
from api.views.invite import InviteViewSet
from api.views.report import ReportViewSet


router = SimpleRouter()

router.register(r"v1/user", APIUserViewSet)
router.register(r"v1/group", GroupViewSet)
router.register(r"v1/wallet", WalletViewSet)
router.register(r"v1/incomecategory", IncomeCategoryViewSet)
router.register(r"v1/expensecategory", ExpenseCategoryViewSet)
router.register(r"v1/income", IncomeViewSet)
router.register(r"v1/expense", ExpenseViewSet)
router.register(r"v1/transfer", TransferViewSet)
router.register(r"v1/currency", CurrencyViewSet)
router.register(r"v1/currencyrate", CurrencyRateViewSet)
router.register(r"v1/report", ReportViewSet, basename="report")

docs_urlpatterns = [
    path("v1/schema/", SpectacularAPIView.as_view(), name="schema"),
    path(
        "v1/schema/swagger-ui/",
        SpectacularSwaggerView.as_view(url_name="schema"),
        name="swagger-ui",
    ),
    path(
        "v1/schema/redoc/",
        SpectacularRedocView.as_view(url_name="schema"),
        name="redoc",
    ),
    path("v1/invite/", InviteViewSet.as_view({"post": "invite"})),
]

auth_urlpatterns = [
    path("v1/auth/get_token/", get_token),
    path("v1/auth/signup/", signup),
    path("v1/auth/signin/", csrf_exempt(signin)),
]

urlpatterns = router.urls + docs_urlpatterns + auth_urlpatterns



from moneybox.settings import PRIMITIVE


def encrypt_token(token):
    ciphertext = PRIMITIVE.encrypt_deterministically(token, b"associated_data")
    return ciphertext


def decrypt_ciphertext(ciphertext):
    decrypted_text = PRIMITIVE.decrypt_deterministically(ciphertext, b"associated_data")
    return decrypted_text.decode()



from django.db import models

from wallet.models.group import Group
from wallet.models.mixins import TimestampMixin, SafeDeletionMixin, SafeDeletionManager
from users.models import APIUser


class Wallet(TimestampMixin, SafeDeletionMixin):
    name = models.CharField(
        max_length=255,
        verbose_name="Name",
        help_text="Name of the wallet",
        db_index=True,
    )
    balance = models.DecimalField(
        max_digits=10,
        decimal_places=2,
        verbose_name="Balance",
        help_text="Current balance of the wallet",
    )
    created_by = models.ForeignKey(
        APIUser,
        on_delete=models.CASCADE,
        verbose_name="Created by",
        help_text="Owner of the wallet",
    )
    group = models.ForeignKey(
        Group,
        on_delete=models.CASCADE,
        verbose_name="Group",
        help_text="Group that the wallet belongs to",
        db_index=True,
    )
    currency = models.ForeignKey(
        "Currency",
        on_delete=models.CASCADE,
        verbose_name="Currency",
        help_text="Currency of the wallet",
        db_index=True,
    )
    objects = SafeDeletionManager()

    class Meta:
        verbose_name = "Wallet"
        verbose_name_plural = "Wallets"



from django.contrib import admin

from wallet.models.expense import ExpenseCategory, Expense


@admin.register(ExpenseCategory)
class ExpenseCategoryAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "name",
        "group",
        "created_at",
        "updated_at",
    )
    list_filter = (
        "created_at",
        "updated_at",
    )
    search_fields = ("name",)
    date_hierarchy = "created_at"


@admin.register(Expense)
class ExpenseAdmin(admin.ModelAdmin):
    list_display = (
        "id",
        "amount",
        "category",
        "comment",
        "created_by",
        "wallet",
        "group",
        "created_at",
        "updated_at",
    )
    list_filter = (
        "created_at",
        "updated_at",
    )
    date_hierarchy = "created_at"



# coding: utf-8

"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from openapi_client.models.invite import Invite  # noqa: E501

class TestInvite(unittest.TestCase):
    """Invite unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Invite:
        """Test Invite
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Invite`
        """
        model = Invite()  # noqa: E501
        if include_optional:
            return Invite(
                id = 56,
                group = openapi_client.models.group.Group(
                    id = 56, 
                    created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    is_deleted = True, 
                    members = [
                        56
                        ], ),
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                is_deleted = True,
                invite_code = -2147483648,
                expires_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f')
            )
        else:
            return Invite(
                id = 56,
                group = openapi_client.models.group.Group(
                    id = 56, 
                    created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'), 
                    is_deleted = True, 
                    members = [
                        56
                        ], ),
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                invite_code = -2147483648,
                expires_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
        )
        """

    def testInvite(self):
        """Test Invite"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



from django.contrib.auth.models import AnonymousUser
from django.core import exceptions
from rest_framework import permissions


class IsAuthenticated(permissions.BasePermission):
    """Allows access only to authenticated users."""

    def has_permission(self, request, view):
        token = request.auth
        return bool(request.user and token)


class IsAdminOrReadOnly(permissions.BasePermission):
    """Allows unsafe methods to be performed only by administrators."""

    def has_permission(self, request, view):
        if request.method in permissions.SAFE_METHODS:
            return True
        if isinstance(request.user, AnonymousUser):
            return False
        try:
            request.user.admin_user
        except exceptions.ObjectDoesNotExist:
            return False
        return bool(request.user and request.user.admin_user.is_staff)



from rest_framework.exceptions import APIException


class RateNotExist(Exception):
    def __init__(self, message="Wrong Data input or this rate doesn`t exist"):
        self.message = message
        super().__init__(self.message)


class ReportAPIException(APIException):
    def __init__(self, detail, status_code=None):
        super().__init__(detail)
        self.status_code = status_code



import sys

from uuid import uuid4

from django.contrib.auth.models import AbstractUser, UserManager
from django.db import models

from api.encryption import decrypt_ciphertext, encrypt_token
from wallet.models.mixins import TimestampMixin, SafeDeletionMixin


class APIUser(TimestampMixin, SafeDeletionMixin):
    token = models.BinaryField()

    class Meta:
        verbose_name = "API User"
        verbose_name_plural = "API Users"

    def __str__(self):
        return f"API User with ID {self.pk}"


class CustomUserManager(UserManager):
    def create_superuser(self, username, password=None, **extra_fields):
        from api.utils import add_defaults

        token_new = str(uuid4())
        token_db = encrypt_token(token_new.encode())
        new_api_user = APIUser.objects.create(token=token_db)

        user = self.create_user(
            username=username, password=password, api_user=new_api_user, is_staff=True, is_superuser=True
        )
        user.save(using=self._db)
        token = decrypt_ciphertext(user.api_user.token)
        sys.stdout.write(f"API user token: {token}\n")
        add_defaults(user=new_api_user)
        return user


class User(AbstractUser):
    api_user = models.OneToOneField(APIUser, on_delete=models.CASCADE, related_name="admin_user")

    REQUIRED_FIELDS = []

    class Meta:
        verbose_name = "Administrator"
        verbose_name_plural = "Administrators"

    objects = CustomUserManager()



# coding: utf-8

# flake8: noqa
"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


# import models into model package
from openapi_client.models.api_user import APIUser
from openapi_client.models.currency import Currency
from openapi_client.models.currency_rate import CurrencyRate
from openapi_client.models.expense import Expense
from openapi_client.models.expense_category import ExpenseCategory
from openapi_client.models.group import Group
from openapi_client.models.income import Income
from openapi_client.models.income_category import IncomeCategory
from openapi_client.models.invite import Invite
from openapi_client.models.paginated_api_user_list import PaginatedAPIUserList
from openapi_client.models.paginated_currency_list import PaginatedCurrencyList
from openapi_client.models.paginated_currency_rate_list import PaginatedCurrencyRateList
from openapi_client.models.paginated_expense_category_list import PaginatedExpenseCategoryList
from openapi_client.models.paginated_expense_list import PaginatedExpenseList
from openapi_client.models.paginated_group_list import PaginatedGroupList
from openapi_client.models.paginated_income_category_list import PaginatedIncomeCategoryList
from openapi_client.models.paginated_income_list import PaginatedIncomeList
from openapi_client.models.paginated_transfer_list import PaginatedTransferList
from openapi_client.models.paginated_wallet_list import PaginatedWalletList
from openapi_client.models.patched_api_user import PatchedAPIUser
from openapi_client.models.patched_currency import PatchedCurrency
from openapi_client.models.patched_currency_rate import PatchedCurrencyRate
from openapi_client.models.patched_expense import PatchedExpense
from openapi_client.models.patched_expense_category import PatchedExpenseCategory
from openapi_client.models.patched_group import PatchedGroup
from openapi_client.models.patched_income import PatchedIncome
from openapi_client.models.patched_income_category import PatchedIncomeCategory
from openapi_client.models.patched_transfer import PatchedTransfer
from openapi_client.models.patched_wallet import PatchedWallet
from openapi_client.models.report import Report
from openapi_client.models.signup import Signup
from openapi_client.models.transfer import Transfer
from openapi_client.models.wallet import Wallet



from drf_spectacular.utils import extend_schema
from rest_framework.viewsets import ModelViewSet

from api.serializers import ExpenseCategorySerializer, ExpenseSerializer
from wallet.models.expense import ExpenseCategory, Expense


@extend_schema(tags=["Expenses"])
class ExpenseCategoryViewSet(ModelViewSet):
    queryset = ExpenseCategory.objects.order_by("pk")
    serializer_class = ExpenseCategorySerializer


@extend_schema(tags=["Expenses"])
class ExpenseViewSet(ModelViewSet):
    queryset = Expense.objects.order_by("pk")
    serializer_class = ExpenseSerializer



from django.db import models

from wallet.models.mixins import TimestampMixin, SafeDeletionMixin, SafeDeletionManager
from core.defs.exeptions import RateNotExist


class FiatCurrency:
    RUB = "RUB"


class CryptoCurrency:
    BITCOIN = "bitcoin"
    TON = "ton"
    ETHEREUM = "ethereum"
    USDC = "usdc"


class CurrencyType:
    FIAT = "fiat"
    CRYPTO = "crypto"

    CHOICES = [
        (FIAT, FIAT),
        (CRYPTO, CRYPTO),
    ]


class Currency(TimestampMixin, SafeDeletionMixin):
    code = models.CharField(
        max_length=10,
        unique=True,
        verbose_name="Currency Code",
        help_text="The unique code for this currency",
    )
    name = models.CharField(
        max_length=255,
        verbose_name="Currency Name",
        help_text="The name of the currency, e.g. US Dollar",
    )
    type = models.CharField(
        max_length=255,
        verbose_name="Currency Type",
        help_text="The type of the currency, e.g. fiat or crypto",
        choices=CurrencyType.CHOICES,
        default=CurrencyType.FIAT,
    )
    objects = SafeDeletionManager()

    class Meta:
        verbose_name = "Currency"
        verbose_name_plural = "Currencies"

    def __str__(self):
        return f"{self.code} {self.name}"


class CurrencyRate(TimestampMixin, SafeDeletionMixin):
    currency = models.ForeignKey(
        Currency,
        related_name="rate",
        on_delete=models.CASCADE,
        verbose_name="Currency",
        help_text="Currency",
        null=True,
        blank=True,
    )
    rate = models.DecimalField(
        max_digits=12,
        decimal_places=5,
        verbose_name="Exchange Rate",
        help_text="The rate at which the source currency" "can be exchanged for the target currency.",
    )
    objects = SafeDeletionManager()

    class Meta:
        verbose_name = "Currency rate"
        verbose_name_plural = "Currency rates"
        get_latest_by = ("created_at",)

    def __repr__(self):
        return f"{self.currency} is {self.rate}"

    @classmethod
    def get_exchange_rate(cls, currency_from, currency_to, date):
        first_value = cls.objects.filter(
            currency=currency_from, created_at__day=date.day, created_at__month=date.month, created_at__year=date.year
        ).latest()
        second_value = cls.objects.filter(
            currency=currency_to, created_at__day=date.day, created_at__month=date.month, created_at__year=date.year
        ).latest()
        if first_value and second_value:
            return round((first_value.rate / second_value.rate), 4)
        raise RateNotExist



from django.contrib import admin

from wallet.models.invite import Invite


@admin.register(Invite)
class InviteAdmin(admin.ModelAdmin):
    list_display = ("id", "invite_code", "group", "created_at", "updated_at")
    list_filter = ("created_at", "updated_at")
    search_fields = ("name", "group")
    date_hierarchy = "created_at"



from django.utils import timezone

from wallet.models.mixins import TimestampMixin, SafeDeletionMixin, SafeDeletionManager

from django.db import models
from wallet.models.group import Group


class Invite(TimestampMixin, SafeDeletionMixin):
    invite_code = models.IntegerField()
    group = models.ForeignKey(Group, on_delete=models.CASCADE, related_name="invites")
    expires_at = models.DateTimeField()
    objects = SafeDeletionManager()

    class Meta:
        constraints = (
            models.UniqueConstraint(
                fields=(
                    "invite_code",
                    "group",
                ),
                name="unique_invite",
            ),
        )
        verbose_name = "Invite"
        verbose_name_plural = "Invites"

    @property
    def is_expired(self):
        return self.expires_at < timezone.now()



# coding: utf-8

"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.api.reports_api import ReportsApi  # noqa: E501


class TestReportsApi(unittest.TestCase):
    """ReportsApi unit test stubs"""

    def setUp(self) -> None:
        self.api = ReportsApi()  # noqa: E501

    def tearDown(self) -> None:
        pass

    def test_report_html_retrieve(self) -> None:
        """Test case for report_html_retrieve

        """
        pass

    def test_report_list(self) -> None:
        """Test case for report_list

        """
        pass


if __name__ == '__main__':
    unittest.main()



from django.db import models


class TimestampMixin(models.Model):
    created_at = models.DateTimeField(
        auto_now_add=True,
        verbose_name="Created at",
        help_text="Date and time of creation",
        db_index=True,
    )
    updated_at = models.DateTimeField(
        auto_now=True,
        verbose_name="Updated at",
        help_text="Date and time of last update",
        db_index=True,
    )

    class Meta:
        abstract = True
        verbose_name = "Timestamp Mixin"
        verbose_name_plural = "Timestamp Mixins"


class SafeDeletionMixin(models.Model):
    is_deleted = models.BooleanField(default=False)

    class Meta:
        abstract = True

    def delete(self, using=None, keep_parents=False):
        self.is_deleted = True
        self.save()

    def hard_delete(self):
        super().delete()


class SafeDeletionManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(is_deleted=False)



from rest_framework.serializers import ModelSerializer

from api.serializers import GroupSerializer
from wallet.models.invite import Invite


class InviteSerializer(ModelSerializer):
    group = GroupSerializer()

    class Meta:
        model = Invite
        fields = "__all__"



from drf_spectacular.utils import extend_schema
from rest_framework.viewsets import ModelViewSet

from api.serializers import IncomeCategorySerializer, IncomeSerializer
from wallet.models.income import IncomeCategory, Income


@extend_schema(tags=["Incomes"])
class IncomeCategoryViewSet(ModelViewSet):
    queryset = IncomeCategory.objects.order_by("pk")
    serializer_class = IncomeCategorySerializer


@extend_schema(tags=["Incomes"])
class IncomeViewSet(ModelViewSet):
    queryset = Income.objects.order_by("pk")
    serializer_class = IncomeSerializer



from django.db import models, transaction

from wallet.models.group import Group
from wallet.models.mixins import TimestampMixin, SafeDeletionMixin, SafeDeletionManager
from wallet.models.wallet import Wallet
from users.models import APIUser


class ExpenseCategory(TimestampMixin, SafeDeletionMixin):
    name = models.CharField(
        max_length=255,
        verbose_name="Name",
        help_text="The name of the expense category",
        db_index=True,
    )
    group = models.ForeignKey(
        Group,
        on_delete=models.CASCADE,
        verbose_name="Group",
        help_text="The group this expense category belongs to",
        db_index=True,
    )
    created_by = models.ForeignKey(
        APIUser,
        on_delete=models.CASCADE,
        verbose_name="Created by",
        help_text="The user who created this expense category",
    )
    objects = SafeDeletionManager()

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = "Expense Category"
        verbose_name_plural = "Expense Categories"


class Expense(TimestampMixin, SafeDeletionMixin):
    amount = models.DecimalField(max_digits=10, decimal_places=2, verbose_name="Amount of expense")
    category = models.ForeignKey(
        ExpenseCategory,
        on_delete=models.CASCADE,
        verbose_name="Expense category",
        db_index=True,
    )
    comment = models.CharField(max_length=255, blank=True, null=True, verbose_name="Comment on expense")
    created_by = models.ForeignKey(APIUser, on_delete=models.CASCADE, verbose_name="User who made the expense")
    wallet = models.ForeignKey(
        Wallet,
        on_delete=models.CASCADE,
        verbose_name="Wallet used for the expense",
        db_index=True,
    )
    group = models.ForeignKey(
        Group,
        on_delete=models.CASCADE,
        verbose_name="Group related to the expense",
        db_index=True,
    )
    objects = SafeDeletionManager()

    class Meta:
        verbose_name = "Expense"
        verbose_name_plural = "Expenses"

    @transaction.atomic
    def save(self, *args, **kwargs):
        super().save(*args, **kwargs)
        self.wallet.balance -= self.amount
        self.wallet.save()



# coding: utf-8

"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from openapi_client.models.report import Report  # noqa: E501

class TestReport(unittest.TestCase):
    """Report unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Report:
        """Test Report
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Report`
        """
        model = Report()  # noqa: E501
        if include_optional:
            return Report(
                balance = 1.337,
                total_incomes = 1.337,
                total_expenses = 1.337,
                income_expense_ratio = 1.337,
                category_incomes = {
                    'key' : null
                    },
                category_expenses = {
                    'key' : null
                    }
            )
        else:
            return Report(
                balance = 1.337,
                total_incomes = 1.337,
                total_expenses = 1.337,
                income_expense_ratio = 1.337,
                category_incomes = {
                    'key' : null
                    },
                category_expenses = {
                    'key' : null
                    },
        )
        """

    def testReport(self):
        """Test Report"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import re  # noqa: F401
import io
import warnings

from pydantic import validate_arguments, ValidationError

from typing_extensions import Annotated
from pydantic import Field, StrictInt, StrictStr

from typing import Optional

from openapi_client.models.currency import Currency
from openapi_client.models.currency_rate import CurrencyRate
from openapi_client.models.paginated_currency_list import PaginatedCurrencyList
from openapi_client.models.paginated_currency_rate_list import PaginatedCurrencyRateList
from openapi_client.models.patched_currency import PatchedCurrency
from openapi_client.models.patched_currency_rate import PatchedCurrencyRate

from openapi_client.api_client import ApiClient
from openapi_client.api_response import ApiResponse
from openapi_client.exceptions import (  # noqa: F401
    ApiTypeError,
    ApiValueError
)


class CurrencyApi:
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    def __init__(self, api_client=None) -> None:
        if api_client is None:
            api_client = ApiClient.get_default()
        self.api_client = api_client

    @validate_arguments
    def currency_create(self, currency : Currency, **kwargs) -> Currency:  # noqa: E501
        """currency_create  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_create(currency, async_req=True)
        >>> result = thread.get()

        :param currency: (required)
        :type currency: Currency
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Currency
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currency_create_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currency_create_with_http_info(currency, **kwargs)  # noqa: E501

    @validate_arguments
    def currency_create_with_http_info(self, currency : Currency, **kwargs) -> ApiResponse:  # noqa: E501
        """currency_create  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_create_with_http_info(currency, async_req=True)
        >>> result = thread.get()

        :param currency: (required)
        :type currency: Currency
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Currency, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'currency'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currency_create" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['currency'] is not None:
            _body_params = _params['currency']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json', 'application/x-www-form-urlencoded', 'multipart/form-data']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '201': "Currency",
        }

        return self.api_client.call_api(
            '/api/v1/currency/', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currency_destroy(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency.")], **kwargs) -> None:  # noqa: E501
        """currency_destroy  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_destroy(id, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency. (required)
        :type id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currency_destroy_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currency_destroy_with_http_info(id, **kwargs)  # noqa: E501

    @validate_arguments
    def currency_destroy_with_http_info(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency.")], **kwargs) -> ApiResponse:  # noqa: E501
        """currency_destroy  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_destroy_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency. (required)
        :type id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = [
            'id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currency_destroy" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/api/v1/currency/{id}/', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currency_list(self, page : Annotated[Optional[StrictInt], Field(description="A page number within the paginated result set.")] = None, search : Annotated[Optional[StrictStr], Field(description="A search term.")] = None, **kwargs) -> PaginatedCurrencyList:  # noqa: E501
        """currency_list  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_list(page, search, async_req=True)
        >>> result = thread.get()

        :param page: A page number within the paginated result set.
        :type page: int
        :param search: A search term.
        :type search: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PaginatedCurrencyList
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currency_list_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currency_list_with_http_info(page, search, **kwargs)  # noqa: E501

    @validate_arguments
    def currency_list_with_http_info(self, page : Annotated[Optional[StrictInt], Field(description="A page number within the paginated result set.")] = None, search : Annotated[Optional[StrictStr], Field(description="A search term.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """currency_list  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_list_with_http_info(page, search, async_req=True)
        >>> result = thread.get()

        :param page: A page number within the paginated result set.
        :type page: int
        :param search: A search term.
        :type search: str
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PaginatedCurrencyList, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'page',
            'search'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currency_list" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('page') is not None:  # noqa: E501
            _query_params.append(('page', _params['page']))

        if _params.get('search') is not None:  # noqa: E501
            _query_params.append(('search', _params['search']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '200': "PaginatedCurrencyList",
        }

        return self.api_client.call_api(
            '/api/v1/currency/', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currency_partial_update(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency.")], patched_currency : Optional[PatchedCurrency] = None, **kwargs) -> Currency:  # noqa: E501
        """currency_partial_update  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_partial_update(id, patched_currency, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency. (required)
        :type id: int
        :param patched_currency:
        :type patched_currency: PatchedCurrency
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Currency
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currency_partial_update_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currency_partial_update_with_http_info(id, patched_currency, **kwargs)  # noqa: E501

    @validate_arguments
    def currency_partial_update_with_http_info(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency.")], patched_currency : Optional[PatchedCurrency] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """currency_partial_update  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_partial_update_with_http_info(id, patched_currency, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency. (required)
        :type id: int
        :param patched_currency:
        :type patched_currency: PatchedCurrency
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Currency, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id',
            'patched_currency'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currency_partial_update" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['patched_currency'] is not None:
            _body_params = _params['patched_currency']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json', 'application/x-www-form-urlencoded', 'multipart/form-data']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '200': "Currency",
        }

        return self.api_client.call_api(
            '/api/v1/currency/{id}/', 'PATCH',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currency_retrieve(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency.")], **kwargs) -> Currency:  # noqa: E501
        """currency_retrieve  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_retrieve(id, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency. (required)
        :type id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Currency
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currency_retrieve_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currency_retrieve_with_http_info(id, **kwargs)  # noqa: E501

    @validate_arguments
    def currency_retrieve_with_http_info(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency.")], **kwargs) -> ApiResponse:  # noqa: E501
        """currency_retrieve  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_retrieve_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency. (required)
        :type id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Currency, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currency_retrieve" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '200': "Currency",
        }

        return self.api_client.call_api(
            '/api/v1/currency/{id}/', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currency_update(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency.")], currency : Currency, **kwargs) -> Currency:  # noqa: E501
        """currency_update  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_update(id, currency, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency. (required)
        :type id: int
        :param currency: (required)
        :type currency: Currency
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: Currency
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currency_update_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currency_update_with_http_info(id, currency, **kwargs)  # noqa: E501

    @validate_arguments
    def currency_update_with_http_info(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency.")], currency : Currency, **kwargs) -> ApiResponse:  # noqa: E501
        """currency_update  # noqa: E501

        List of currencies or add new currency  # noqa: E501
        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currency_update_with_http_info(id, currency, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency. (required)
        :type id: int
        :param currency: (required)
        :type currency: Currency
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(Currency, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id',
            'currency'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currency_update" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['currency'] is not None:
            _body_params = _params['currency']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json', 'application/x-www-form-urlencoded', 'multipart/form-data']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '200': "Currency",
        }

        return self.api_client.call_api(
            '/api/v1/currency/{id}/', 'PUT',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currencyrate_create(self, currency_rate : CurrencyRate, **kwargs) -> CurrencyRate:  # noqa: E501
        """currencyrate_create  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_create(currency_rate, async_req=True)
        >>> result = thread.get()

        :param currency_rate: (required)
        :type currency_rate: CurrencyRate
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CurrencyRate
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currencyrate_create_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currencyrate_create_with_http_info(currency_rate, **kwargs)  # noqa: E501

    @validate_arguments
    def currencyrate_create_with_http_info(self, currency_rate : CurrencyRate, **kwargs) -> ApiResponse:  # noqa: E501
        """currencyrate_create  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_create_with_http_info(currency_rate, async_req=True)
        >>> result = thread.get()

        :param currency_rate: (required)
        :type currency_rate: CurrencyRate
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CurrencyRate, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'currency_rate'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currencyrate_create" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['currency_rate'] is not None:
            _body_params = _params['currency_rate']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json', 'application/x-www-form-urlencoded', 'multipart/form-data']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '201': "CurrencyRate",
        }

        return self.api_client.call_api(
            '/api/v1/currencyrate/', 'POST',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currencyrate_destroy(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency rate.")], **kwargs) -> None:  # noqa: E501
        """currencyrate_destroy  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_destroy(id, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency rate. (required)
        :type id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currencyrate_destroy_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currencyrate_destroy_with_http_info(id, **kwargs)  # noqa: E501

    @validate_arguments
    def currencyrate_destroy_with_http_info(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency rate.")], **kwargs) -> ApiResponse:  # noqa: E501
        """currencyrate_destroy  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_destroy_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency rate. (required)
        :type id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: None
        """

        _params = locals()

        _all_params = [
            'id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currencyrate_destroy" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {}

        return self.api_client.call_api(
            '/api/v1/currencyrate/{id}/', 'DELETE',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currencyrate_get_rate_retrieve(self, **kwargs) -> CurrencyRate:  # noqa: E501
        """currencyrate_get_rate_retrieve  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_get_rate_retrieve(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CurrencyRate
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currencyrate_get_rate_retrieve_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currencyrate_get_rate_retrieve_with_http_info(**kwargs)  # noqa: E501

    @validate_arguments
    def currencyrate_get_rate_retrieve_with_http_info(self, **kwargs) -> ApiResponse:  # noqa: E501
        """currencyrate_get_rate_retrieve  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_get_rate_retrieve_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CurrencyRate, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currencyrate_get_rate_retrieve" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '200': "CurrencyRate",
        }

        return self.api_client.call_api(
            '/api/v1/currencyrate/get_rate/', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currencyrate_list(self, page : Annotated[Optional[StrictInt], Field(description="A page number within the paginated result set.")] = None, **kwargs) -> PaginatedCurrencyRateList:  # noqa: E501
        """currencyrate_list  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_list(page, async_req=True)
        >>> result = thread.get()

        :param page: A page number within the paginated result set.
        :type page: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: PaginatedCurrencyRateList
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currencyrate_list_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currencyrate_list_with_http_info(page, **kwargs)  # noqa: E501

    @validate_arguments
    def currencyrate_list_with_http_info(self, page : Annotated[Optional[StrictInt], Field(description="A page number within the paginated result set.")] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """currencyrate_list  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_list_with_http_info(page, async_req=True)
        >>> result = thread.get()

        :param page: A page number within the paginated result set.
        :type page: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(PaginatedCurrencyRateList, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'page'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currencyrate_list" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}

        # process the query parameters
        _query_params = []
        if _params.get('page') is not None:  # noqa: E501
            _query_params.append(('page', _params['page']))

        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '200': "PaginatedCurrencyRateList",
        }

        return self.api_client.call_api(
            '/api/v1/currencyrate/', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currencyrate_partial_update(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency rate.")], patched_currency_rate : Optional[PatchedCurrencyRate] = None, **kwargs) -> CurrencyRate:  # noqa: E501
        """currencyrate_partial_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_partial_update(id, patched_currency_rate, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency rate. (required)
        :type id: int
        :param patched_currency_rate:
        :type patched_currency_rate: PatchedCurrencyRate
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CurrencyRate
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currencyrate_partial_update_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currencyrate_partial_update_with_http_info(id, patched_currency_rate, **kwargs)  # noqa: E501

    @validate_arguments
    def currencyrate_partial_update_with_http_info(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency rate.")], patched_currency_rate : Optional[PatchedCurrencyRate] = None, **kwargs) -> ApiResponse:  # noqa: E501
        """currencyrate_partial_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_partial_update_with_http_info(id, patched_currency_rate, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency rate. (required)
        :type id: int
        :param patched_currency_rate:
        :type patched_currency_rate: PatchedCurrencyRate
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CurrencyRate, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id',
            'patched_currency_rate'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currencyrate_partial_update" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['patched_currency_rate'] is not None:
            _body_params = _params['patched_currency_rate']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json', 'application/x-www-form-urlencoded', 'multipart/form-data']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '200': "CurrencyRate",
        }

        return self.api_client.call_api(
            '/api/v1/currencyrate/{id}/', 'PATCH',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currencyrate_retrieve(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency rate.")], **kwargs) -> CurrencyRate:  # noqa: E501
        """currencyrate_retrieve  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_retrieve(id, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency rate. (required)
        :type id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CurrencyRate
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currencyrate_retrieve_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currencyrate_retrieve_with_http_info(id, **kwargs)  # noqa: E501

    @validate_arguments
    def currencyrate_retrieve_with_http_info(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency rate.")], **kwargs) -> ApiResponse:  # noqa: E501
        """currencyrate_retrieve  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_retrieve_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency rate. (required)
        :type id: int
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CurrencyRate, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currencyrate_retrieve" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '200': "CurrencyRate",
        }

        return self.api_client.call_api(
            '/api/v1/currencyrate/{id}/', 'GET',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))

    @validate_arguments
    def currencyrate_update(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency rate.")], currency_rate : CurrencyRate, **kwargs) -> CurrencyRate:  # noqa: E501
        """currencyrate_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_update(id, currency_rate, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency rate. (required)
        :type id: int
        :param currency_rate: (required)
        :type currency_rate: CurrencyRate
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _request_timeout: timeout setting for this request.
               If one number provided, it will be total request
               timeout. It can also be a pair (tuple) of
               (connection, read) timeouts.
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: CurrencyRate
        """
        kwargs['_return_http_data_only'] = True
        if '_preload_content' in kwargs:
            message = "Error! Please call the currencyrate_update_with_http_info method with `_preload_content` instead and obtain raw data from ApiResponse.raw_data"  # noqa: E501
            raise ValueError(message)
        return self.currencyrate_update_with_http_info(id, currency_rate, **kwargs)  # noqa: E501

    @validate_arguments
    def currencyrate_update_with_http_info(self, id : Annotated[StrictInt, Field(..., description="A unique integer value identifying this Currency rate.")], currency_rate : CurrencyRate, **kwargs) -> ApiResponse:  # noqa: E501
        """currencyrate_update  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True

        >>> thread = api.currencyrate_update_with_http_info(id, currency_rate, async_req=True)
        >>> result = thread.get()

        :param id: A unique integer value identifying this Currency rate. (required)
        :type id: int
        :param currency_rate: (required)
        :type currency_rate: CurrencyRate
        :param async_req: Whether to execute the request asynchronously.
        :type async_req: bool, optional
        :param _preload_content: if False, the ApiResponse.data will
                                 be set to none and raw_data will store the
                                 HTTP response body without reading/decoding.
                                 Default is True.
        :type _preload_content: bool, optional
        :param _return_http_data_only: response data instead of ApiResponse
                                       object with status code, headers, etc
        :type _return_http_data_only: bool, optional
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :param _request_auth: set to override the auth_settings for an a single
                              request; this effectively ignores the authentication
                              in the spec for a single request.
        :type _request_auth: dict, optional
        :type _content_type: string, optional: force content-type for the request
        :return: Returns the result object.
                 If the method is called asynchronously,
                 returns the request thread.
        :rtype: tuple(CurrencyRate, status_code(int), headers(HTTPHeaderDict))
        """

        _params = locals()

        _all_params = [
            'id',
            'currency_rate'
        ]
        _all_params.extend(
            [
                'async_req',
                '_return_http_data_only',
                '_preload_content',
                '_request_timeout',
                '_request_auth',
                '_content_type',
                '_headers'
            ]
        )

        # validate the arguments
        for _key, _val in _params['kwargs'].items():
            if _key not in _all_params:
                raise ApiTypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method currencyrate_update" % _key
                )
            _params[_key] = _val
        del _params['kwargs']

        _collection_formats = {}

        # process the path parameters
        _path_params = {}
        if _params['id']:
            _path_params['id'] = _params['id']


        # process the query parameters
        _query_params = []
        # process the header parameters
        _header_params = dict(_params.get('_headers', {}))
        # process the form parameters
        _form_params = []
        _files = {}
        # process the body parameter
        _body_params = None
        if _params['currency_rate'] is not None:
            _body_params = _params['currency_rate']

        # set the HTTP header `Accept`
        _header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # set the HTTP header `Content-Type`
        _content_types_list = _params.get('_content_type',
            self.api_client.select_header_content_type(
                ['application/json', 'application/x-www-form-urlencoded', 'multipart/form-data']))
        if _content_types_list:
                _header_params['Content-Type'] = _content_types_list

        # authentication setting
        _auth_settings = ['APIAuthentication']  # noqa: E501

        _response_types_map = {
            '200': "CurrencyRate",
        }

        return self.api_client.call_api(
            '/api/v1/currencyrate/{id}/', 'PUT',
            _path_params,
            _query_params,
            _header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            response_types_map=_response_types_map,
            auth_settings=_auth_settings,
            async_req=_params.get('async_req'),
            _return_http_data_only=_params.get('_return_http_data_only'),  # noqa: E501
            _preload_content=_params.get('_preload_content', True),
            _request_timeout=_params.get('_request_timeout'),
            collection_formats=_collection_formats,
            _request_auth=_params.get('_request_auth'))



import json
import os
import tink

from dotenv import load_dotenv
from tink import cleartext_keyset_handle
from tink import daead
from pathlib import Path

daead.register()
load_dotenv()


BASE_DIR = Path(__file__).resolve().parent.parent

SECRET_KEY = os.getenv("DJANGO_SECRET", "secret")

DEBUG = os.getenv("DJANGO_DEBUG", "True") == "True"

ALLOWED_HOSTS = ["*"]

INSTALLED_APPS = [
    "users",
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "rest_framework",
    "api",
    "wallet",
    "core",
    "drf_spectacular",
    "drf_generators",
    "django_celery_beat",
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "moneybox.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
        },
    },
]

WSGI_APPLICATION = "moneybox.wsgi.application"

LOCAL_ENV = "local"

ENV = os.getenv("ENV", LOCAL_ENV)

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.postgresql",
        "NAME": os.getenv("POSTGRES_DB"),
        "USER": os.getenv("POSTGRES_USER"),
        "PASSWORD": os.getenv("POSTGRES_PASSWORD"),
        "HOST": os.getenv("POSTGRES_DB_HOST", "db"),
        "PORT": os.getenv("POSTGRES_DB_PORT", "5342"),
    }
}

AUTH_USER_MODEL = "users.User"

AUTH_HEADER = "Authorization"

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]

LANGUAGE_CODE = "en-us"

TIME_ZONE = "UTC"

USE_I18N = True

USE_TZ = True

STATIC_URL = "/static/"
STATIC_ROOT = os.path.join(BASE_DIR, "static")

DEFAULT_AUTO_FIELD = "django.db.models.BigAutoField"

REST_FRAMEWORK = {
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.PageNumberPagination",
    "PAGE_SIZE": 15,
    "DEFAULT_SCHEMA_CLASS": "drf_spectacular.openapi.AutoSchema",
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "api.authentication.APIAuthentication",
    ],
    "DEFAULT_PERMISSION_CLASSES": [
        "api.permissions.IsAuthenticated",
    ],
}

LOGGING = {
    "version": 1,
    "disable_existing_loggers": False,
    "handlers": {
        "console": {
            "class": "logging.StreamHandler",
        },
    },
    "root": {
        "handlers": ["console"],
        "level": os.getenv("DJANGO_LOG_LEVEL", "INFO"),
    },
}


SPECTACULAR_SETTINGS = {
    "TITLE": "MoneyBox",
    "DESCRIPTION": (
        "MoneyBox will help you better understand your finances, "
        "improve financial literacy, and achieve financial goals."
    ),
    "VERSION": "1.0.0",
    "SERVE_INCLUDE_SCHEMA": False,
    "DEFAULT_AUTHENTICATION_CLASSES": [
        "api.authentication.APIAuthentication",
    ],
    "SERVE_PERMISSIONS": ["rest_framework.permissions.AllowAny"],
    "SERVE_AUTHENTICATION": None,
}
# celery
CELERY_BEAT_SCHEDULER = os.getenv("CELERY_BEAT_SCHEDULER", "django_celery_beat.schedulers:DatabaseScheduler")
CELERY_BROKER_URL = os.getenv("CELERY_BROKER_URL")
# cbr
CBR_TIMEOUT = 10
CBR_URL = "https://cbr.ru/scripts/XML_daily.asp"
# coingecko
COINGECKO_TIMEOUT = 10
COINGECKO_URL = "https://api.coingecko.com/api/v3/simple/price?ids={crypto_currencies}&vs_currencies={fiat_currency}"
# encoding
PRE_KEYSET = {
    "key": [
        {
            "keyData": {
                "keyMaterialType": os.getenv("KEY_MATERIAL_TYPE", "SYMMETRIC"),
                "typeUrl": os.getenv("TYPE_URL", "type.googleapis.com/google.crypto.tink.AesSivKey"),
                "value": os.getenv(
                    "VALUE", "EkDseJb9CSmxFJJ66dRrwQXN+ToxcBxJC/GPwhFfTYHREMfgTX2t52RpDQ6u29TdintRxJm8RLaMMvmjmDao1Lpu"
                ),
            },
            "keyId": os.getenv("KEY_ID", 2021474508),
            "outputPrefixType": os.getenv("OUTPUT_PREFIX_TYPE", "TINK"),
            "status": os.getenv("STATUS", "ENABLED"),
        }
    ],
    "primaryKeyId": os.getenv("PRIMARY_KEY_ID", 2021474508),
}
KEYSET = json.dumps(PRE_KEYSET, indent=4)
KEYSET_HANDLE = cleartext_keyset_handle.read(tink.JsonKeysetReader(KEYSET))
PRIMITIVE = KEYSET_HANDLE.primitive(daead.DeterministicAead)



from datetime import datetime
from django.utils import timezone

from django.db.models import Sum, Max

from core.defs import constants
from core.defs.datetime import convert_date_to_datetime_format
from wallet.models.currency import Currency, FiatCurrency
from wallet.models.expense import ExpenseCategory
from wallet.models.group import Group
from wallet.models.income import IncomeCategory
from wallet.models.wallet import Wallet


def get_start_end_dates(start_date=None, end_date=None):
    if not start_date or not end_date:
        today = timezone.now().date()
        start_of_day = timezone.make_aware(datetime.combine(today, datetime.min.time()))
        end_of_day = timezone.make_aware(datetime.combine(today, datetime.max.time()))
    else:
        start_of_day = timezone.make_aware(datetime.combine(start_date, datetime.min.time()))
        end_of_day = timezone.make_aware(datetime.combine(end_date, datetime.max.time()))

    return start_of_day, end_of_day


def get_category_data(profile, model, start_date=None, end_date=None):
    category_data = (
        model.objects.filter(created_by=profile, created_at__range=get_start_end_dates(start_date, end_date))
        .values("category__name")
        .annotate(total_expenses=Sum("amount"), created_at=Max("created_at"))
        .values("category__name", "total_expenses", "created_at")
    )

    return [{**x, "created_at": convert_date_to_datetime_format(x["created_at"])} for x in category_data]


def get_total_data(group, model, start_date=None, end_date=None):
    total_data_per = (
        getattr(group, model)
        .filter(created_at__range=get_start_end_dates(start_date, end_date))
        .aggregate(total_data=Sum("amount"))
        .get("total_data")
        or 0
    )

    total_data = getattr(group, model).aggregate(total_data=Sum("amount")).get("total_data") or 0

    return total_data_per, total_data


def add_defaults(user):
    """Add expense/income/wallet categories."""

    group = Group.objects.create()
    group.members.add(user)
    currency, _ = Currency.objects.get_or_create(code=FiatCurrency.RUB, name=constants.RUB_NAME)

    expense_categories = [
        ExpenseCategory(name=expense_category, group=group, created_by=user)
        for expense_category in constants.DEFAULT_EXPENSE_CATEGORY
    ]
    ExpenseCategory.objects.bulk_create(expense_categories)

    income_categories = [
        ExpenseCategory(name=income_category, group=group, created_by=user)
        for income_category in constants.DEFAULT_INCOME_CATEGORY
    ]
    IncomeCategory.objects.bulk_create(income_categories)

    wallets = [
        Wallet(
            name=wallet,
            balance=0,
            group=group,
            created_by=user,
            currency=currency,
        )
        for wallet in constants.WALLET_LIST
    ]
    Wallet.objects.bulk_create(wallets)



from rest_framework.serializers import ModelSerializer

from wallet.models.income import IncomeCategory, Income


class IncomeCategorySerializer(ModelSerializer):
    class Meta:
        model = IncomeCategory
        fields = "__all__"


class IncomeSerializer(ModelSerializer):
    class Meta:
        model = Income
        fields = "__all__"



from django.utils import timezone

from clients.currency.cbr.cbr import cbr_client
from clients.currency.coingecko.coingecko import coingecko_client
from moneybox.celery import app
from wallet.models.currency import Currency, CurrencyRate, CurrencyType
from wallet.models.invite import Invite


@app.task
def get_exchange_rates():
    """Delivering currency rates from cbr.ru"""
    currencies = cbr_client.get_currencies_rates()
    for code, code_data in currencies.items():
        currency, _ = Currency.objects.get_or_create(
            code=code,
            name=code_data.get("name", code),
        )
        rate = code_data.get("value") / code_data.get("nominal")
        CurrencyRate.objects.create(currency=currency, rate=rate)


@app.task
def delete_expired_invites():
    """Removes expired invites"""
    expired_invite_codes = Invite.objects.filter(expires_at__lt=timezone.now())
    expired_invite_codes.delete()


@app.task
def update_crypto_exchange_rates():
    """Delivering cryptocurrency rates from coingecko.com"""
    rates = coingecko_client.get_rates()
    for currency, rate in rates.items():
        currency, _ = Currency.objects.get_or_create(
            code=currency.upper()[:4],
            name=currency,
            type=CurrencyType.CRYPTO,
        )
        CurrencyRate.objects.create(currency=currency, rate=round(rate, 4))



from drf_spectacular.utils import extend_schema
from rest_framework.viewsets import ModelViewSet

from api.serializers import APIUserSerializer
from users.models import APIUser


@extend_schema(tags=["Users"])
class APIUserViewSet(ModelViewSet):
    queryset = APIUser.objects.order_by("pk")
    serializer_class = APIUserSerializer



# coding: utf-8

"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from openapi_client.models.paginated_api_user_list import PaginatedAPIUserList  # noqa: E501

class TestPaginatedAPIUserList(unittest.TestCase):
    """PaginatedAPIUserList unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> PaginatedAPIUserList:
        """Test PaginatedAPIUserList
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `PaginatedAPIUserList`
        """
        model = PaginatedAPIUserList()  # noqa: E501
        if include_optional:
            return PaginatedAPIUserList(
                count = 123,
                next = 'http://api.example.org/accounts/?page=4',
                previous = 'http://api.example.org/accounts/?page=2',
                results = [
                    openapi_client.models.api_user.APIUser(
                        token = 'YQ==', )
                    ]
            )
        else:
            return PaginatedAPIUserList(
        )
        """

    def testPaginatedAPIUserList(self):
        """Test PaginatedAPIUserList"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()



from django.contrib import admin
from django.urls import path, include
from django.conf.urls.static import static
from moneybox.settings import DEBUG, STATIC_URL, STATIC_ROOT

from core.urls import urlpatterns as core_urlpatterns

urlpatterns = [
    path("admin/", admin.site.urls),
    path("api/", include("api.urls")),
]
urlpatterns += core_urlpatterns
if DEBUG:
    urlpatterns += static(STATIC_URL, document_root=STATIC_ROOT)



# coding: utf-8

"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.api.currency_api import CurrencyApi  # noqa: E501


class TestCurrencyApi(unittest.TestCase):
    """CurrencyApi unit test stubs"""

    def setUp(self) -> None:
        self.api = CurrencyApi()  # noqa: E501

    def tearDown(self) -> None:
        pass

    def test_currency_create(self) -> None:
        """Test case for currency_create

        """
        pass

    def test_currency_destroy(self) -> None:
        """Test case for currency_destroy

        """
        pass

    def test_currency_list(self) -> None:
        """Test case for currency_list

        """
        pass

    def test_currency_partial_update(self) -> None:
        """Test case for currency_partial_update

        """
        pass

    def test_currency_retrieve(self) -> None:
        """Test case for currency_retrieve

        """
        pass

    def test_currency_update(self) -> None:
        """Test case for currency_update

        """
        pass

    def test_currencyrate_create(self) -> None:
        """Test case for currencyrate_create

        """
        pass

    def test_currencyrate_destroy(self) -> None:
        """Test case for currencyrate_destroy

        """
        pass

    def test_currencyrate_get_rate_retrieve(self) -> None:
        """Test case for currencyrate_get_rate_retrieve

        """
        pass

    def test_currencyrate_list(self) -> None:
        """Test case for currencyrate_list

        """
        pass

    def test_currencyrate_partial_update(self) -> None:
        """Test case for currencyrate_partial_update

        """
        pass

    def test_currencyrate_retrieve(self) -> None:
        """Test case for currencyrate_retrieve

        """
        pass

    def test_currencyrate_update(self) -> None:
        """Test case for currencyrate_update

        """
        pass


if __name__ == '__main__':
    unittest.main()



from api.views.auth import get_token, signin, signup
from api.views.currency import CurrencyViewSet, CurrencyRateViewSet
from api.views.expense import ExpenseCategoryViewSet, ExpenseViewSet
from api.views.group import GroupViewSet
from api.views.income import IncomeViewSet, IncomeCategoryViewSet
from api.views.user import APIUserViewSet
from api.views.transfer import TransferViewSet
from api.views.wallet import WalletViewSet

__all__ = [
    "APIUserViewSet",
    "CurrencyViewSet",
    "CurrencyRateViewSet",
    "ExpenseCategoryViewSet",
    "ExpenseViewSet",
    "get_token",
    "GroupViewSet",
    "IncomeCategoryViewSet",
    "IncomeViewSet",
    "signin",
    "signup",
    "TransferViewSet",
    "WalletViewSet",
]



from api.serializers.currency import CurrencySerializer, CurrencyRateSerializer
from api.serializers.group import GroupSerializer
from api.serializers.expense import ExpenseSerializer, ExpenseCategorySerializer
from api.serializers.income import IncomeCategorySerializer, IncomeSerializer
from api.serializers.user import APIUserSerializer, SignupSerializer
from api.serializers.transfer import TransferSerializer
from api.serializers.wallet import WalletSerializer

__all__ = [
    "APIUserSerializer",
    "CurrencySerializer",
    "CurrencyRateSerializer",
    "ExpenseSerializer",
    "ExpenseCategorySerializer",
    "GroupSerializer",
    "IncomeSerializer",
    "IncomeCategorySerializer",
    "SignupSerializer",
    "TransferSerializer",
    "WalletSerializer",
]



from django.apps import AppConfig


class ApiConfig(AppConfig):
    default_auto_field = "django.db.models.BigAutoField"
    name = "api"

    def ready(self):
        import api.schema  # noqa: F401



from drf_spectacular.utils import extend_schema
from rest_framework.viewsets import ModelViewSet

from api.serializers import WalletSerializer
from wallet.models.wallet import Wallet


@extend_schema(tags=["Wallets"])
class WalletViewSet(ModelViewSet):
    queryset = Wallet.objects.order_by("pk")
    serializer_class = WalletSerializer



from uuid import uuid4

from django.db import transaction
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.decorators import api_view, permission_classes
from rest_framework.permissions import AllowAny
from rest_framework.response import Response

from api.encryption import decrypt_ciphertext, encrypt_token
from api.serializers import APIUserSerializer, SignupSerializer
from api.utils import add_defaults
from moneybox.settings import AUTH_HEADER
from users.models import APIUser
from wallet.models.invite import Invite


@extend_schema(request=SignupSerializer, responses=APIUserSerializer, tags=["Auth"])
@api_view(("POST",))
@permission_classes((AllowAny,))
@transaction.atomic
def signup(request):
    serializer = SignupSerializer(data=request.data)
    if not serializer.is_valid():
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    invite_code = serializer.validated_data.get("invite_code")
    token = str(uuid4())
    token_db = encrypt_token(token.encode())
    user = APIUser.objects.create(token=token_db)
    token_for_user = decrypt_ciphertext(user.token)

    if invite_code:
        group_invite = Invite.objects.filter(invite_code=invite_code).first()
        group = group_invite.group
        group.members.add(user)
        group_invite.delete()
        return Response({"token": token_for_user}, status=status.HTTP_201_CREATED)

    add_defaults(user=user)

    return Response({"token": token_for_user}, status=status.HTTP_201_CREATED)


@extend_schema(request=APIUserSerializer, responses=None, tags=["Auth"])
@api_view(("POST",))
@permission_classes((AllowAny,))
def signin(request):
    token_passed = request.data["token"]
    token_db = encrypt_token(token_passed.encode())
    if not APIUser.objects.filter(token=token_db).exists():
        return Response({"error": "no such token exists"}, status.HTTP_401_UNAUTHORIZED)
    return Response(status=status.HTTP_200_OK)


@extend_schema(request=None, responses=APIUserSerializer, tags=["Auth"])
@api_view(("GET",))
def get_token(request):
    auth_header = request.headers[AUTH_HEADER]
    token_header = auth_header.split(" ")[1]
    token_db = encrypt_token(token_header.encode())
    if not APIUser.objects.filter(token=token_db).exists():
        return Response({"error": "no such token exists"}, status.HTTP_401_UNAUTHORIZED)
    return Response({"token": token_header}, status.HTTP_200_OK)



# coding: utf-8

"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest

from openapi_client.api.invites_api import InvitesApi  # noqa: E501


class TestInvitesApi(unittest.TestCase):
    """InvitesApi unit test stubs"""

    def setUp(self) -> None:
        self.api = InvitesApi()  # noqa: E501

    def tearDown(self) -> None:
        pass

    def test_invite_create(self) -> None:
        """Test case for invite_create

        """
        pass


if __name__ == '__main__':
    unittest.main()



from django.http import HttpResponse
from django.shortcuts import get_object_or_404
from drf_spectacular.utils import extend_schema
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response

from api.serializers.report import ReportSerializer
from api.utils import get_category_data, get_start_end_dates, get_total_data
from core.defs.chart_generator import generate_charts
from core.defs.datetime import convert_date_to_standart_format
from core.defs.exeptions import ReportAPIException
from users.models import APIUser
from wallet.models.expense import Expense
from wallet.models.income import Income


@extend_schema(tags=["Reports"])
class ReportViewSet(viewsets.ViewSet):
    serializer_class = ReportSerializer

    @staticmethod
    def list(request):
        user_profile = ReportViewSet.get_user_profile(request.user)
        queryset = ReportViewSet.get_queryset(user_profile)
        serializer = ReportViewSet.serializer_class(queryset, context={"request": request})

        report_data = serializer.data
        return Response({"report_data": report_data})

    @action(detail=False, methods=["get"])
    def html(self, request):
        user_profile = ReportViewSet.get_user_profile(request.user)
        queryset = ReportViewSet.get_queryset(user_profile)
        serializer = ReportViewSet.serializer_class(queryset, context={"request": request})

        report_data = serializer.data

        # Getting the selected dates from the request parameters
        start_date = request.query_params.get("start_date")
        end_date = request.query_params.get("end_date")

        # Use the get_start_end_dates() function to get the start and end date
        start_of_day, end_of_day = get_start_end_dates(start_date, end_date)

        if start_date == end_date:
            x_axis_data = [convert_date_to_standart_format(start_of_day)]
        else:
            x_axis_data = [convert_date_to_standart_format(start_of_day), convert_date_to_standart_format(end_of_day)]

        chart_html = generate_charts(x_axis_data, report_data)

        return HttpResponse(chart_html, content_type="text/html")

    @staticmethod
    def get_user_profile(user):
        profile = get_object_or_404(APIUser, token=user)
        return profile

    @staticmethod
    def get_total_incomes(group, start_date=None, end_date=None):
        try:
            return get_total_data(group, "income_set", start_date, end_date)
        except Exception as e:
            raise ReportAPIException(detail=f"Error occurred while calculating total incomes: {e}")

    @staticmethod
    def get_total_expenses(group, start_date=None, end_date=None):
        try:
            return get_total_data(group, "expense_set", start_date, end_date)
        except Exception as e:
            raise ReportAPIException(detail=f"Error occurred while calculating total expenses: {e}")

    @staticmethod
    def get_income_expense_ratio(total_incomes_per, total_expenses_per):
        try:
            if total_expenses_per != 0:
                income_expense_ratio = total_incomes_per / total_expenses_per
            else:
                income_expense_ratio = 0
            return income_expense_ratio
        except Exception as e:
            raise ReportAPIException(detail=f"Error occurred while calculating income-expense ratio: {e}")

    @staticmethod
    def get_category_incomes(profile, start_date=None, end_date=None):
        try:
            return get_category_data(profile, Income, start_date, end_date)
        except Exception as e:
            raise ReportAPIException(detail=f"Error occurred while fetching category incomes: {e}")

    @staticmethod
    def get_category_expenses(profile, start_date=None, end_date=None):
        try:
            return get_category_data(profile, Expense, start_date, end_date)
        except Exception as e:
            raise ReportAPIException(detail=f"Error occurred while fetching category expenses: {e}")

    @staticmethod
    def get_queryset(profile, start_date=None, end_date=None):
        total_incomes_per, total_incomes = ReportViewSet.get_total_incomes(profile, start_date, end_date)
        total_expenses_per, total_expenses = ReportViewSet.get_total_expenses(profile, start_date, end_date)

        income_expense_ratio = ReportViewSet.get_income_expense_ratio(total_incomes_per, total_expenses_per)

        queryset = {
            "balance": total_incomes - total_expenses,
            "total_incomes": total_incomes_per,
            "total_expenses": total_expenses_per,
            "income_expense_ratio": income_expense_ratio,
            "category_incomes": ReportViewSet.get_category_incomes(profile, start_date, end_date),
            "category_expenses": ReportViewSet.get_category_expenses(profile, start_date, end_date),
        }

        return queryset



# coding: utf-8

"""
    MoneyBox

    MoneyBox will help you better understand your finances, improve financial literacy, and achieve financial goals.

    The version of the OpenAPI document: 1.0.0
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


import unittest
import datetime

from openapi_client.models.group import Group  # noqa: E501

class TestGroup(unittest.TestCase):
    """Group unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def make_instance(self, include_optional) -> Group:
        """Test Group
            include_option is a boolean, when False only required
            params are included, when True both required and
            optional params are included """
        # uncomment below to create an instance of `Group`
        """
        model = Group()  # noqa: E501
        if include_optional:
            return Group(
                id = 56,
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                is_deleted = True,
                members = [
                    56
                    ]
            )
        else:
            return Group(
                id = 56,
                created_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                updated_at = datetime.datetime.strptime('2013-10-20 19:20:30.00', '%Y-%m-%d %H:%M:%S.%f'),
                members = [
                    56
                    ],
        )
        """

    def testGroup(self):
        """Test Group"""
        # inst_req_only = self.make_instance(include_optional=False)
        # inst_req_and_optional = self.make_instance(include_optional=True)

if __name__ == '__main__':
    unittest.main()
